@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (c) 2016 by the fifechan team                               *
+ *   Copyright (c) 2017-2019 by the fifechan team                               *
  *   https://github.com/fifengine/fifechan                                 *
  *   This file is part of fifechan.                                        *
  *                                                                         *
@@ -308,6 +308,20 @@ namespace fcn
          */
         virtual void handleMouseWheelMovedUp(const MouseInput& mouseInput);
 
+        /**
+         * Handles mouse wheel moved right input.
+         *
+         * @param mouseInput The mouse input to handle.
+         */
+        virtual void handleMouseWheelMovedRight(const MouseInput& mouseInput);
+
+        /**
+         * Handles mouse wheel moved left input.
+         *
+         * @param mouseInput The mouse input to handle.
+         */
+        virtual void handleMouseWheelMovedLeft(const MouseInput& mouseInput);
+
         /**
          * Handles mouse released input.
          *
