@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (c) 2016 by the fifechan team                               *
+ *   Copyright (c) 2017-2019 by the fifechan team                               *
  *   https://github.com/fifengine/fifechan                                 *
  *   This file is part of fifechan.                                        *
  *                                                                         *
@@ -366,6 +366,11 @@ namespace fcn
 
         void setDimension(const Rectangle& dimension);
 
+        virtual void resizeToContent(bool recursiv=true);
+
+        virtual void adjustSize();
+
+        virtual void expandContent(bool recursiv=true);
 
         // Inherited from MouseListener
 
@@ -379,6 +384,10 @@ namespace fcn
 
         virtual void mouseWheelMovedDown(MouseEvent& mouseEvent);
 
+        virtual void mouseWheelMovedRight(MouseEvent& mouseEvent);
+
+        virtual void mouseWheelMovedLeft(MouseEvent& mouseEvent);
+
     protected:
         /**
          * Draws the background of the scroll area, that is
