@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (c) 2017 by the fifechan team                               *
+ *   Copyright (c) 2017-2019 by the fifechan team                               *
  *   https://github.com/fifengine/fifechan                                 *
  *   This file is part of fifechan.                                        *
  *                                                                         *
@@ -81,16 +81,17 @@ namespace fcn
         : mHasMouse(false),
           mKeyPressed(false),
           mMousePressed(false),
+          mState(true),
           mAlignment(Graphics::Center),
-          mSpacing(4)
-    {
+          mXOffset(1),
+          mYOffset(1) {
         setFocusable(true);
         adjustSize();
-        setFrameSize(1);
 
         addMouseListener(this);
         addKeyListener(this);
         addFocusListener(this);
+        addWidgetListener(this);
     }
 
     Button::Button(const std::string& caption)
@@ -98,208 +99,205 @@ namespace fcn
               mHasMouse(false),
               mKeyPressed(false),
               mMousePressed(false),
+              mState(true),
               mAlignment(Graphics::Center),
-              mSpacing(4)
-    {
+              mXOffset(1),
+              mYOffset(1) {
         setFocusable(true);
         adjustSize();
-        setFrameSize(1);
 
         addMouseListener(this);
         addKeyListener(this);
         addFocusListener(this);
+        addWidgetListener(this);
     }
 
-    void Button::setCaption(const std::string& caption)
-    {
+    Button::~Button() {
+
+    }
+
+    void Button::setCaption(const std::string& caption) {
         mCaption = caption;
+        adjustSize();
     }
 
-    const std::string& Button::getCaption() const
-    {
+    const std::string& Button::getCaption() const {
         return mCaption;
     }
 
-    void Button::setAlignment(Graphics::Alignment alignment)
-    {
+    void Button::setActive(bool state) {
+        mState = state;
+    }
+
+    bool Button::isActive() const {
+        return mState;
+    }
+
+    void Button::setAlignment(Graphics::Alignment alignment) {
         mAlignment = alignment;
     }
 
-    Graphics::Alignment Button::getAlignment() const
-    {
+    Graphics::Alignment Button::getAlignment() const {
         return mAlignment;
     }
 
-    void Button::setSpacing(unsigned int spacing)
-    {
-        mSpacing = spacing;
+    void Button::setDownXOffset(int offset) {
+        mXOffset = offset;
     }
 
-    unsigned int Button::getSpacing() const
-    {
-      return mSpacing;
+    int Button::getDownXOffset() const {
+        return mXOffset;
     }
 
-    void Button::draw(Graphics* graphics)
-    {
+    void Button::setDownYOffset(int offset) {
+        mYOffset = offset;
+    }
+
+    int Button::getDownYOffset() const {
+        return mYOffset;
+    }
+
+    void Button::setDownOffset(int x, int y) {
+        mXOffset = x;
+        mYOffset = y;
+    }
+
+    void Button::fontChanged() {
+        adjustSize();
+    }
+
+    void Button::draw(Graphics* graphics) {
+        bool active = isFocused();
         Color faceColor = getBaseColor();
-        Color highlightColor, shadowColor;
-        int alpha = getBaseColor().a;
+        if (active && ((getSelectionMode() & Widget::Selection_Background) == Widget::Selection_Background)) {
+            faceColor = getSelectionColor();
+        }
+        int alpha = faceColor.a;
 
-        if (isPressed())
-        {
+        if (isPressed()) {
             faceColor = faceColor - 0x303030;
             faceColor.a = alpha;
-            highlightColor = faceColor - 0x303030;
-            highlightColor.a = alpha;
-            shadowColor = faceColor + 0x303030;
-            shadowColor.a = alpha;
         }
-        else
-        {
-            highlightColor = faceColor + 0x303030;
-            highlightColor.a = alpha;
-            shadowColor = faceColor - 0x303030;
-            shadowColor.a = alpha;
+        if (!isActive()) {
+            int color = static_cast<int>(faceColor.r * 0.3 + faceColor.g * 0.59 + faceColor.b * 0.11);
+            faceColor.r = color;
+            faceColor.g = color;
+            faceColor.b = color;
         }
 
         graphics->setColor(faceColor);
-        graphics->fillRectangle(1, 1, getDimension().width-1, getHeight() - 1);
+        Rectangle offsetRec(getBorderSize(), getBorderSize(), 2 * getBorderSize(), 2 * getBorderSize());
+        graphics->fillRectangle(offsetRec.x, offsetRec.y, getWidth() - offsetRec.width, getHeight() - offsetRec.height);
 
-        graphics->setColor(highlightColor);
-        graphics->drawLine(0, 0, getWidth() - 1, 0);
-        graphics->drawLine(0, 1, 0, getHeight() - 1);
-
-        graphics->setColor(shadowColor);
-        graphics->drawLine(getWidth() - 1, 1, getWidth() - 1, getHeight() - 1);
-        graphics->drawLine(1, getHeight() - 1, getWidth() - 1, getHeight() - 1);
-
-        graphics->setColor(getForegroundColor());
+        if (getBorderSize() > 0) {
+            if (active && (getSelectionMode() & Widget::Selection_Border) == Widget::Selection_Border) {
+                drawSelectionFrame(graphics);
+            } else {
+                drawBorder(graphics);
+            }
+        }
 
         int textX;
-        int textY = getHeight() / 2 - getFont()->getHeight() / 2;
-
-        switch (getAlignment())
-        {
+        int textY = offsetRec.y + getPaddingTop() + (getHeight() - offsetRec.height - getPaddingTop() - getPaddingBottom() - getFont()->getHeight()) / 2;
+        switch (getAlignment()) {
           case Graphics::Left:
-              textX = mSpacing;
+              textX = offsetRec.x + getPaddingLeft();
               break;
           case Graphics::Center:
-              textX = getWidth() / 2;
+              textX = offsetRec.x + getPaddingLeft() + (getWidth() - offsetRec.width - getPaddingLeft() - getPaddingRight()) / 2;
               break;
           case Graphics::Right:
-              textX = getWidth() - mSpacing;
+              textX = getWidth() - offsetRec.x - getPaddingRight();
               break;
           default:
               throw FCN_EXCEPTION("Unknown alignment.");
         }
 
+        // set font and color
         graphics->setFont(getFont());
-
-        if (isPressed())
-        {
-            graphics->drawText(getCaption(), textX + 1, textY + 1, getAlignment());
-        }
-        else
-        {
+        graphics->setColor(getForegroundColor());
+        if (isPressed()) {
+            graphics->drawText(getCaption(), textX + getDownXOffset(), textY + getDownYOffset(), getAlignment());
+        } else {
             graphics->drawText(getCaption(), textX, textY, getAlignment());
-
-            if (isFocused())
-            {
-                graphics->drawRectangle(2, 2, getWidth() - 4,
-                                        getHeight() - 4);
-            }
         }
     }
 
-    void Button::adjustSize()
-    {
-        setWidth(getFont()->getWidth(mCaption) + 2*mSpacing);
-        setHeight(getFont()->getHeight() + 2*mSpacing);
+    void Button::resizeToContent(bool recursiv) {
+        adjustSize();
     }
 
-    bool Button::isPressed() const
-    {
-        if (mMousePressed)
-        {
+    void Button::adjustSize() {
+        int w = getFont()->getWidth(mCaption) + 2 * getBorderSize() + getPaddingLeft() + getPaddingRight();
+        int h = getFont()->getHeight() + 2 * getBorderSize() + getPaddingTop() + getPaddingBottom();
+        setSize(w, h);
+    }
+
+    bool Button::isPressed() const {
+        if (mMousePressed) {
             return mHasMouse;
         }
-        else
-        {
-            return mKeyPressed;
-        }
+        return mKeyPressed;
     }
 
-    void Button::mousePressed(MouseEvent& mouseEvent)
-    {
-        if (mouseEvent.getButton() == MouseEvent::Left)
-        {
+    void Button::mousePressed(MouseEvent& mouseEvent) {
+        if (mouseEvent.getButton() == MouseEvent::Left) {
             mMousePressed = true;
             mouseEvent.consume();
         }
     }
 
-    void Button::mouseExited(MouseEvent& mouseEvent)
-    {
+    void Button::mouseExited(MouseEvent& mouseEvent) {
         mHasMouse = false;
     }
 
-    void Button::mouseEntered(MouseEvent& mouseEvent)
-    {
+    void Button::mouseEntered(MouseEvent& mouseEvent) {
         mHasMouse = true;
     }
 
-    void Button::mouseReleased(MouseEvent& mouseEvent)
-    {
-        if (mouseEvent.getButton() == MouseEvent::Left
-            && mMousePressed
-            && mHasMouse)
-        {
+    void Button::mouseReleased(MouseEvent& mouseEvent) {
+        if (mouseEvent.getButton() == MouseEvent::Left && mMousePressed && mHasMouse) {
             mMousePressed = false;
             distributeActionEvent();
             mouseEvent.consume();
-        }
-        else if (mouseEvent.getButton() == MouseEvent::Left)
-        {
+        } else if (mouseEvent.getButton() == MouseEvent::Left) {
             mMousePressed = false;
             mouseEvent.consume();
         }
     }
 
-    void Button::mouseDragged(MouseEvent& mouseEvent)
-    {
+    void Button::mouseDragged(MouseEvent& mouseEvent) {
         mouseEvent.consume();
     }
 
-    void Button::keyPressed(KeyEvent& keyEvent)
-    {
+    void Button::keyPressed(KeyEvent& keyEvent) {
         Key key = keyEvent.getKey();
 
-        if (key.getValue() == Key::Enter
-            || key.getValue() == Key::Space)
-        {
+        if (key.getValue() == Key::Enter || key.getValue() == Key::Space) {
             mKeyPressed = true;
             keyEvent.consume();
         }
     }
 
-    void Button::keyReleased(KeyEvent& keyEvent)
-    {
+    void Button::keyReleased(KeyEvent& keyEvent) {
         Key key = keyEvent.getKey();
 
-        if ((key.getValue() == Key::Enter
-             || key.getValue() == Key::Space)
-            && mKeyPressed)
-        {
+        if ((key.getValue() == Key::Enter || key.getValue() == Key::Space) && mKeyPressed) {
             mKeyPressed = false;
             distributeActionEvent();
             keyEvent.consume();
         }
     }
 
-    void Button::focusLost(const Event& event)
-    {
+    void Button::focusLost(const Event& event) {
         mMousePressed = false;
         mKeyPressed = false;
+        mHasMouse = false;
+    }
+
+    void Button::ancestorHidden(const Event& e) {
+        mMousePressed = false;
+        mKeyPressed = false;
+        mHasMouse = false;
     }
 }
