@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (c) 2017 by the fifechan team                               *
+ *   Copyright (c) 2017-2019 by the fifechan team                               *
  *   https://github.com/fifengine/fifechan                                 *
  *   This file is part of fifechan.                                        *
  *                                                                         *
@@ -464,6 +464,21 @@ namespace fcn
         mScrollArea->setPosition(0, 0);
     }
 
+    void DropDown::resizeToContent(bool recursiv) {
+        if (mScrollArea != NULL) {
+            mScrollArea->resizeToContent();
+        }
+
+        if (mListBox != NULL) {
+            mScrollArea->resizeToContent();
+        }
+        adjustHeight();
+    }
+
+    void DropDown::adjustSize() {
+        adjustHeight();
+    }
+
     void DropDown::dropDown()
     {
         if (!mDroppedDown)
