@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (c) 2017 by the fifechan team                               *
+ *   Copyright (c) 2017-2019 by the fifechan team                               *
  *   https://github.com/fifengine/fifechan                                 *
  *   This file is part of fifechan.                                        *
  *                                                                         *
@@ -82,7 +82,7 @@ namespace fcn
         mScaleEnd = scaleEnd;
 
         setFocusable(true);
-        setFrameSize(1);
+        setBorderSize(1);
         setOrientation(Horizontal);
         setValue(0);
         setStepLength(scaleEnd / 10);
@@ -100,7 +100,7 @@ namespace fcn
         mScaleEnd = scaleEnd;
 
         setFocusable(true);
-        setFrameSize(1);
+        setBorderSize(1);
         setOrientation(Horizontal);
         setValue(scaleStart);
         setStepLength((scaleEnd  - scaleStart)/ 10);
